'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

require('core-js/modules/es.array.includes');
require('core-js/modules/es.string.includes');
var warning = _interopDefault(require('tiny-warning'));
require('core-js/modules/es.array.map');
require('core-js/modules/es.object.assign');
require('core-js/modules/es.object.entries');
require('core-js/modules/es.array.is-array');
require('core-js/modules/es.function.name');
var vueRouter = require('vue-router');
require('core-js/modules/es.array.join');

function createPath(literals) {
  var path = '';

  for (var i = 0; i < (arguments.length <= 1 ? 0 : arguments.length - 1); i++) {
    var l = literals[i];
    var p = i + 1 < 1 || arguments.length <= i + 1 ? undefined : arguments[i + 1];
    path += l + ":" + p;
     warning(!l.includes(':'), "vue-routider: `:` should not be included in createPath argument. (" + l + ")") ;
     warning(!p.includes(':'), "vue-routider: `:` should not be included in createPath argument. (" + p + ")") ;
  }

  var lastL = literals[literals.length - 1];
  path += lastL;
   warning(!lastL.includes(':'), "vue-routider: `:` should not be included in createPath argument. (" + lastL + ")") ;
  return path;
}
function createPaths() {
  for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
    args[_key] = arguments[_key];
  }

  return args;
}
var pathToString = function pathToString(path) {
  return path;
}; // eslint-disable-next-line @typescript-eslint/no-explicit-any

var pathsToString = function pathsToString(paths) {
  return paths;
};

var pathToPathAndAlias = function pathToPathAndAlias(path) {
  if (!Array.isArray(path)) return {
    path: pathToString(path)
  };

  if (path.length <= 0) {
    throw new Error('Path property in Route of RoutiderOptions must have at least 1 path.');
  }

  var pathStrings = pathsToString(path); // eslint-disable-next-line @typescript-eslint/no-non-null-assertion

  var pathProp = pathStrings.shift();
  return {
    path: pathProp,
    alias: pathStrings
  };
};
var createRoute = function createRoute(route) {
  return route;
};

var routiderOptionsToRouterOptions = function routiderOptionsToRouterOptions(options) {
  var routes = Object.entries(options.routes).map(function (_ref) {
    var name = _ref[0],
        route = _ref[1];
    return Object.assign({}, route, {
      name: name
    }, pathToPathAndAlias(route.path));
  });
  return Object.assign({}, options, {
    routes: routes
  });
};

var createRoutiderRouter = function createRoutiderRouter(router) {
  var isRouteName = function isRouteName(location, name) {
    return name === location.name;
  };

  var getOptionalTypedRoute = function getOptionalTypedRoute(location) {
    return location;
  };

  return Object.assign({}, router, {
    isRouteName: isRouteName,
    getOptionalTypedRoute: getOptionalTypedRoute
  });
};

var warnIfIncorrectRoute = function warnIfIncorrectRoute(route, name) {
  var _route$matched$;

  if (name === null) return;
  var matchedName = (_route$matched$ = route.matched[0]) === null || _route$matched$ === void 0 ? void 0 : _route$matched$.name;
  if (matchedName === undefined) return;
  var names = Array.isArray(name) ? name : [name];
   warning(names.includes(matchedName), "vue-routider: Incorrect useRoute usage. Path is now matched '" + String(matchedName) + "' but `useRoute([" + names.join(',') + "])` is used.") ;
};

var createRoutider = function createRoutider(options) {
  var routerOptions = routiderOptionsToRouterOptions(options);
  var rawRouter = vueRouter.createRouter(routerOptions);
  var router = createRoutiderRouter(rawRouter);

  var useRouter = function useRouter() {
    return router;
  };

  var useRoute = function useRoute(name) {
    var route = vueRouter.useRoute();

    {
      warnIfIncorrectRoute(route, name);
    }

    return route;
  };

  var onBeforeRouteLeave = function onBeforeRouteLeave(leaveGuard) {
    vueRouter.onBeforeRouteLeave(leaveGuard);
  };

  var onBeforeRouteUpdate = function onBeforeRouteUpdate(updateGuard) {
    vueRouter.onBeforeRouteUpdate(updateGuard);
  };

  return {
    rawRouter: rawRouter,
    router: router,
    useRouter: useRouter,
    useRoute: useRoute,
    onBeforeRouteLeave: onBeforeRouteLeave,
    onBeforeRouteUpdate: onBeforeRouteUpdate
  };
};

exports.createPath = createPath;
exports.createPaths = createPaths;
exports.createRoute = createRoute;
exports.createRoutider = createRoutider;
exports.createRoutiderRouter = createRoutiderRouter;
exports.pathToPathAndAlias = pathToPathAndAlias;
exports.pathToString = pathToString;
exports.pathsToString = pathsToString;
exports.routiderOptionsToRouterOptions = routiderOptionsToRouterOptions;
//# sourceMappingURL=vue-routider.cjs.development.js.map
