import 'core-js/modules/es.array.includes';
import 'core-js/modules/es.string.includes';
import warning from 'tiny-warning';
import 'core-js/modules/es.array.map';
import 'core-js/modules/es.object.assign';
import 'core-js/modules/es.object.entries';
import 'core-js/modules/es.array.is-array';
import 'core-js/modules/es.function.name';
import { createRouter, useRoute, onBeforeRouteLeave, onBeforeRouteUpdate } from 'vue-router';
import 'core-js/modules/es.array.join';

function createPath(literals) {
  var path = '';

  for (var i = 0; i < (arguments.length <= 1 ? 0 : arguments.length - 1); i++) {
    var l = literals[i];
    var p = i + 1 < 1 || arguments.length <= i + 1 ? undefined : arguments[i + 1];
    path += l + ":" + p;
    process.env.NODE_ENV !== "production" ? warning(!l.includes(':'), "vue-routider: `:` should not be included in createPath argument. (" + l + ")") : void 0;
    process.env.NODE_ENV !== "production" ? warning(!p.includes(':'), "vue-routider: `:` should not be included in createPath argument. (" + p + ")") : void 0;
  }

  var lastL = literals[literals.length - 1];
  path += lastL;
  process.env.NODE_ENV !== "production" ? warning(!lastL.includes(':'), "vue-routider: `:` should not be included in createPath argument. (" + lastL + ")") : void 0;
  return path;
}
function createPaths() {
  for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
    args[_key] = arguments[_key];
  }

  return args;
}
var pathToString = function pathToString(path) {
  return path;
}; // eslint-disable-next-line @typescript-eslint/no-explicit-any

var pathsToString = function pathsToString(paths) {
  return paths;
};

var pathToPathAndAlias = function pathToPathAndAlias(path) {
  if (!Array.isArray(path)) return {
    path: pathToString(path)
  };

  if (path.length <= 0) {
    throw new Error('Path property in Route of RoutiderOptions must have at least 1 path.');
  }

  var pathStrings = pathsToString(path); // eslint-disable-next-line @typescript-eslint/no-non-null-assertion

  var pathProp = pathStrings.shift();
  return {
    path: pathProp,
    alias: pathStrings
  };
};
var createRoute = function createRoute(route) {
  return route;
};

var routiderOptionsToRouterOptions = function routiderOptionsToRouterOptions(options) {
  var routes = Object.entries(options.routes).map(function (_ref) {
    var name = _ref[0],
        route = _ref[1];
    return Object.assign({}, route, {
      name: name
    }, pathToPathAndAlias(route.path));
  });
  return Object.assign({}, options, {
    routes: routes
  });
};

var createRoutiderRouter = function createRoutiderRouter(router) {
  var isRouteName = function isRouteName(location, name) {
    return name === location.name;
  };

  var getOptionalTypedRoute = function getOptionalTypedRoute(location) {
    return location;
  };

  return Object.assign({}, router, {
    isRouteName: isRouteName,
    getOptionalTypedRoute: getOptionalTypedRoute
  });
};

var warnIfIncorrectRoute = function warnIfIncorrectRoute(route, name) {
  var _route$matched$;

  if (name === null) return;
  var matchedName = (_route$matched$ = route.matched[0]) === null || _route$matched$ === void 0 ? void 0 : _route$matched$.name;
  if (matchedName === undefined) return;
  var names = Array.isArray(name) ? name : [name];
  process.env.NODE_ENV !== "production" ? warning(names.includes(matchedName), "vue-routider: Incorrect useRoute usage. Path is now matched '" + String(matchedName) + "' but `useRoute([" + names.join(',') + "])` is used.") : void 0;
};

var createRoutider = function createRoutider(options) {
  var routerOptions = routiderOptionsToRouterOptions(options);
  var rawRouter = createRouter(routerOptions);
  var router = createRoutiderRouter(rawRouter);

  var useRouter = function useRouter() {
    return router;
  };

  var useRoute$1 = function useRoute$1(name) {
    var route = useRoute();

    if (process.env.NODE_ENV !== "production") {
      warnIfIncorrectRoute(route, name);
    }

    return route;
  };

  var onBeforeRouteLeave$1 = function onBeforeRouteLeave$1(leaveGuard) {
    onBeforeRouteLeave(leaveGuard);
  };

  var onBeforeRouteUpdate$1 = function onBeforeRouteUpdate$1(updateGuard) {
    onBeforeRouteUpdate(updateGuard);
  };

  return {
    rawRouter: rawRouter,
    router: router,
    useRouter: useRouter,
    useRoute: useRoute$1,
    onBeforeRouteLeave: onBeforeRouteLeave$1,
    onBeforeRouteUpdate: onBeforeRouteUpdate$1
  };
};

export { createPath, createPaths, createRoute, createRoutider, createRoutiderRouter, pathToPathAndAlias, pathToString, pathsToString, routiderOptionsToRouterOptions };
//# sourceMappingURL=vue-routider.esm.js.map
